#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include "dicomlib.h"
#include "phitslib.h"
#include "utllib.h"
/********** Module variables ***********************************************/
int ifendian=0,imendian=0,imatch;
char Fout[1024];
float version=1.00;
void dsp_header(void);

/********** Interface functions ********************************************/

int main()
{
  char Frtdose[1024],Frtimage[1024],Fphitsdose[1024],Fphitsout[1024];
  FILE *fp,*fp1,*fp2;
  double xval[6],fac;
  int ifnorm;
  double xn[3],vnorm;
  int i,ngrid[3],ngridtot;
  int jflag[3];
  char cver[256],cdate[256],ctime[256];
  char dummy[1024],sign0[12]="PHITS2DICOM";
  float *data=NULL,dmax;
  int ipmax[3];
  int idebug=0;

/*--------- Header display -----------------------------------------------*/
  dsp_header();
/*--------- Input --------------------------------------------------------*/
  if(get_fname("Type PHITS2DICOM to run phits2dicom",dummy)!=0)exit(-1);
  if(strncmp(dummy,sign0,11)!=0){
    printf(" ERROR:: signature PHITS2DICOM is not confirmed\n");
    printf(" This input may not be for phits2dicom\n");
    exit(-1);
  }
  if(get_fname("Name of RTdose file with PATH",Frtdose)!=0)exit(-1);
  if(get_fname("Name of RTimage file with PATH",Frtimage)!=0)exit(-1);
  if(get_fname("Name of PHITS dose file with PATH",Fphitsdose)!=0)exit(-1);
  if(get_fname("Name of phits.out file with PATH",Fphitsout)!=0)exit(-1);
  if(get_fname("Name of output file with PATH",Fout)!=0)exit(-1);
  imendian=eval_imendian();
/*--------- Input2 -------------------------------------------------------*/
  if(get_int("Normalization point spcification 0:No 1:Yes",&ifnorm)!=0)exit(-1);
  if(ifnorm!=0){
    if(get_double3("Normalization point [mm]: x, y, z",xn)!=0)exit(-1);
    if(get_double("Normalized value for this point",&vnorm)!=0)exit(-1);
  }
/*--------- Read RTdose --------------------------------------------------*/
  if(open_file(&fp1,Frtdose,"rb")!=0)exit(-1);
  if(check_file(fp1,"RTDOSE")!=0)exit(-1);
  if(open_file(&fp2,Fout,"wb")!=0)exit(-1);
  if(copy_binary(fp1,fp2)!=0)exit(-1);
  if(close_file(fp1)!=0)exit(-1);
  if(close_file(fp2)!=0)exit(-1);
  if(open_file(&fp1,Fout,"r+b")!=0)exit(-1);
  if(idebug!=0){
    printf("--- Original header ---------------------------------------\n");
    get_RTdoseheader(fp1);
    printf("-----------------------------------------------------------\n");
  }
  if(close_file(fp1)!=0)exit(-1);
/*--------- Read RTimage -------------------------------------------------*/
  if(open_file(&fp1,Frtimage,"rb")!=0)exit(-1);
  if(check_file(fp1,"CT")!=0)exit(-1);
  get_RTimageheader(fp1);
  if(close_file(fp1)!=0)exit(-1);
/*--------- Read phits.out -----------------------------------------------*/
  memset(cver,0,256);
  memset(cdate,0,256);
  memset(ctime,0,256);
  if(open_file(&fp,Fphitsout,"r")!=0)exit(-1);
  if(read_phitsout(fp,jflag,cver,cdate,ctime)!=0)exit(-1);
  if(close_file(fp)!=0)exit(-1);
/*--------- Read phits dose file ------------------------------------------*/
  if(open_file(&fp,Fphitsdose,"r")!=0)exit(-1);
  if(read_phitsheader(fp,xval,ngrid)!=0)exit(-1);
  for(i=0;i<6;i++)xval[i]=10.0*xval[i]; /// converting cm to mm
  ngridtot=ngrid[0]*ngrid[1]*ngrid[2];
  data=(float *)malloc(sizeof(float)*ngridtot);
  if(data==NULL){
    printf("ERROR:: allocation of data\n");
    exit(-1);
  }
  rewind(fp);
  if(read_phitsdata(fp,ngrid,data,&dmax,ipmax)!=0)exit(-1);
  if(close_file(fp)!=0)exit(-1);
  /* if(open_file(&fp,"tmp.txt","w")!=0)exit(-1); */
  /* if(write_data(fp,ngrid,data)) */
  /* if(close_file(fp)!=0)exit(-1); */
  printf("--- Created data ------------------------------------------\n");
  printf(" Data points:\n");
  printf("  nx = %d  ny = %d  nz = %d\n",ngrid[0],ngrid[1],ngrid[2]);
  printf(" Data range [mm]:\n");
  printf("  x= %lf : %lf\n",xval[0],xval[1]);
  printf("  y= %lf : %lf\n",xval[2],xval[3]);
  printf("  z= %lf : %lf\n",xval[4],xval[5]);
  printf("-----------------------------------------------------------\n");
/*--------- Create output ------------------------------------------------*/
  if(open_file(&fp1,Fout,"r+b")!=0)exit(-1);
  if(check_file(fp1,"RTDOSE")!=0)exit(-1);
  if(overwrite_RTdoseheader(fp1,xval,ngrid,
			    dmax,ipmax,&fac,ifnorm,data,xn,vnorm,
			    cver,cdate,ctime)!=0)exit(-1);
  if(overwrite_RTdosedata(fp1,fac,ngridtot,data)!=0)exit(-1);
  if(idebug!=0){
    rewind(fp1);
    printf("--- Created header ----------------------------------------\n");
    get_RTdoseheader(fp1);
    printf("-----------------------------------------------------------\n");
  }
  if(close_file(fp1)!=0)exit(-1);
  free(data);

  printf("*** PHITS2DICOM successfully finished *********************\n");
  return(0);
}
/***************************************************************************/
void dsp_header()
{
  printf("***********************************************************\n");
  printf("*     PROGRAM phits2dicom                                 *\n");
  printf("*                                                         *\n");
  printf("*        Created by T. Furuta @ JAEA,                     *\n");
  printf("*              Y. Koba and S. Matsumoto @ NIRS            *\n");
  printf("*                      on 2016/03/03                      *\n");
  printf("*           under collaboration between JAEA and NIRS     *\n");
  printf("*                                                         *\n");
  printf("*---------------------------------------------------------*\n");
  printf("*                   Version = %.2f                        *\n",version);
  printf("*---------------------------------------------------------*\n");
  printf("*                                                         *\n");
  printf("*     Purpose                                             *\n");
  printf("*        Convert 3D dose distribution calculated by PHITS *\n");
  printf("*        to data in DICOM RT-dose format                  *\n");
  printf("*                                                         *\n");
  printf("***********************************************************\n");
  return;
}
