#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include "phitslib.h"

/********** Prototype decllaration *****************************************/
int check_datastart(char *);
int read_axis(char *);
int read_mesh(char *);
int read_meshnum(char *,char *,int *);
int read_xyzparam(char *,char *,int *);

/********** Module functions ***********************************************/
int write_data(FILE *fp, int *ngrid, float *data)
{
  int nx,ny,nz,i,k,ii;
  int imax,irest;
  nx=*ngrid;
  ny=*(ngrid+1);
  nz=*(ngrid+2);
  imax=nx*ny/10;
  irest=(nx*ny)%10;

  for(k=0;k<nz;k++){
    ii=k*nx*ny;
    for(i=0;i<imax;i++){
      fprintf(fp,"%g %g %g %g %g %g %g %g %g %g\n",
	      data[ii],data[ii+1],data[ii+2],data[ii+3],data[ii+4],
	      data[ii+5],data[ii+6],data[ii+7],data[ii+8],data[ii+9]);
      ii+=10;
    }
    if(irest>0){
      for(i=0;i<irest-1;i++){
	fprintf(fp,"%g ",data[ii+i]);
      }
      fprintf(fp,"%g\n",data[ii+irest]);
    }
  }    
  return(0);
}
/*-------------------------------------------------------------------------*/
int read_phitsdata(FILE *fp, int *ngrid, float *data, float *dmax, int *ipmax)
{
  int nx,ny,nz,i,j,k,i0;
  int imax,irest;
  float *tmpdata=NULL;
  char buff[256];
  char *token;

  nx=*ngrid;
  ny=*(ngrid+1);
  nz=*(ngrid+2);
  imax=nx*ny/10;
  *dmax=0.0E0;
  irest=(nx*ny)%10;
  tmpdata=(float *)malloc(sizeof(float)*nx*ny);
  for(k=0;k<nz;k++){
    while(1){
      if(fgets(buff,sizeof(buff),fp)==NULL)return(-1);
      if(strncmp(buff," newpage:",9)==0||
	 strncmp(buff,"#newpage:",9)==0)break;
    }
    while(1){
      if(fgets(buff,sizeof(buff),fp)==NULL)return(-1);
      if(check_datastart(buff)==0)break;
    }
    for(i=0;i<imax;i++){
      if(fgets(buff,sizeof(buff),fp)==NULL)return(-1);
      token=strtok(buff," ");
      for(j=0;j<10;j++){
	tmpdata[i*10+j]=atof(token);
	token=strtok(NULL," ");
      }
    }
    if(irest>0){
      if(fgets(buff,sizeof(buff),fp)==NULL)return(-1);
      token=strtok(buff," ");
      for(j=0;j<irest;j++){
	tmpdata[imax*10+j]=atof(token);
	token=strtok(NULL," ");
      }
    }
    i0=k*nx*ny;
    for(j=0;j<ny;j++){
      for(i=0;i<nx;i++){
	data[i0+(ny-1-j)*nx+i]=tmpdata[j*nx+i];
	/* data[i0+j*nx+i]=tmpdata[j*nx+i]; */
	/* *dmax=fmaxf(tmpdata[j*nx+i],*dmax); */
	if(tmpdata[j*nx+i]>*dmax){
	  *dmax=tmpdata[j*nx+i];
	  *ipmax=i;
	  *(ipmax+1)=ny-1-j;
	  *(ipmax+2)=k;
	}
      }
    }
  }
  free(tmpdata);
  return(0);
}
/*-------------------------------------------------------------------------*/
int read_phitsheader(FILE *fp, double *xval, int *ngrid)
{
  char buff[256],buff0[256],value[256];
  int i,ierr,iflag,jflag[6]={0};
  char *token;
  char cxyz[2],cminmax[4];

  while(fgets(buff,sizeof(buff),fp)!=NULL){
    memset(value,0,256);
    strcpy(buff0,buff);
    token=strtok(buff," ");
    switch(*token){
    case 'x':
      ierr=read_xyzparam(buff0,value,&iflag);
      if(ierr==0&&iflag>0){
	switch(iflag){
	case 1:
	  *xval=atof(value);
	  jflag[0]=1;
	  break;
	case 2:
	  *(xval+1)=atof(value);
	  jflag[1]=1;
	  break;
	}
      }
      break;
    case 'y':
      ierr=read_xyzparam(buff0,value,&iflag);
      if(ierr==0&&iflag>0){
	switch(iflag){
	case 1:
	  *(xval+2)=atof(value);
	  jflag[2]=1;
	  break;
	case 2:
	  *(xval+3)=atof(value);
	  jflag[3]=1;
	  break;
	}
      }
      break;
    case 'z':
      ierr=read_xyzparam(buff0,value,&iflag);
      if(ierr==0&&iflag>0){
	switch(iflag){
	case 1:
	  *(xval+4)=atof(value);
	  jflag[4]=1;
	  break;
	case 2:
	  *(xval+5)=atof(value);
	  jflag[5]=1;
	  break;
	}
      }
      break;
    case 'm':
      ierr=read_mesh(buff0);
      if(ierr==-1)return(-1);
      break;
    case 'a':
      ierr=read_axis(buff0);
      if(ierr==-1)return(-1);
      break;
    case 'n':
      ierr=read_meshnum(buff0,value,&iflag);
      if(ierr==-1)return(-1);
      if(ierr==0){
	switch(iflag){
	case 1:
	  *ngrid=atoi(value);
	case 2:
	  *(ngrid+1)=atoi(value);
	case 3:
	  *(ngrid+2)=atoi(value);
	}
      }
      break;
    case 'h':
      if(check_datastart(buff0)==0){
	//	return(0);
	ierr=1;
	for(i=0;i<6;i++) ierr *= jflag[i];
	if(ierr==0){
	  for(i=0;i<6;i++){
	    if(jflag[i]==0){
	      if(i%2==0){
		strcpy(cminmax,"min\0");
	      }else{
		strcpy(cminmax,"max\0");
	      }
	      switch(i/2){
	      case 0:
		strcpy(cxyz,"x\0");
	      case 1:
		strcpy(cxyz,"y\0");
	      case 2:
		strcpy(cxyz,"z\0");
	      }
	      printf("ERROR: missing %s%s in PHITS dose file\n",cxyz,cminmax);
	    }
	  }
	  return(-1);
	}
	return(0);
      }
      break;
    default:
      break;
    }
  }
  printf("ERROR:: PHITS file ended without sequential data (hc:)\n");
  return(-1);
}
/*-------------------------------------------------------------------------*/
int read_phitsout(FILE *fp,int *jflag, char *cver, char *cdate, char *ctime)
{
  char buff[256],value[256];
  int len,len0;
  char *token;

  while(fgets(buff,sizeof(buff),fp)!=NULL){
    memset(value,0,256);
    token=strtok(buff," ");
    switch(*token){
    case '|':
      token=strtok(NULL," ");
      if(token!=NULL){
	if(strncmp(token,"Version",7)==0){
	  token=strtok(NULL," ");
	  if(strncmp(token,"=",1)==0){
	    token=strtok(NULL," ");
	    len=strlen(token);
	    strncpy(value,token,len);
	    *jflag=1;
	    strncpy(cver,value,len);
	  }
	}
      }
      break;
    case 'S':
      if(strncmp(token,"Starting",8)==0){
	token=strtok(NULL," ");
	if(token!=NULL){
	  if(strncmp(token,"Date",4)==0){
	    token=strtok(NULL," ");
	    if(strncmp(token,"=",1)==0){
	      token=strtok(NULL,"-");
	      len0=0;
	      len=strlen(token);
	      strncpy(value+len0,token,len);
	      token=strtok(NULL,"-");
	      len0+=len;
	      len=strlen(token);
	      strncpy(value+len0,token,len);
	      token=strtok(NULL," ");
	      len0+=len;
	      len=strlen(token);
	      strncpy(value+len0,token,len);
	      len0+=len;
	      *(jflag+1)=1;
	      strncpy(cdate,value,len0-1); // eliminating return code
	    }
	  }else if(strncmp(token,"Time",4)==0){
	    token=strtok(NULL," ");
	    if(strncmp(token,"=",1)==0){
	      token=strtok(NULL," ");
	      len0=0;
	      len=strlen(token)-1;
	      strncpy(value+len0,token,len);
	      token=strtok(NULL," ");
	      len0+=len;
	      len=strlen(token)-1;
	      strncpy(value+len0,token,len);
	      token=strtok(NULL," ");
	      len0+=len;
	      len=strlen(token);
	      strncpy(value+len0,token,len);
	      len0+=len;
	      *(jflag+2)=1;
	      strncpy(ctime,value,len0-1); // eliminating return code
	    }
	  }
	}
      }
      break;
    case '>':
      return(0);
      break;
    default:
      break;
    }
  }
  printf("ERROR:: phits.out ended without '>>> Input Echo'");
  return(-1);
}
/*-------------------------------------------------------------------------*/
int check_datastart(char *buff)
{
  char *token;
  token=strtok(buff," ");
  if(strncmp(token,"hc:",3)==0){
    return(0);
  }
  return(1);
}
/*-------------------------------------------------------------------------*/
int read_meshnum(char *buff, char *value, int *iflag)
{
  char *token;
  int len;
  *iflag=0;
  token=strtok(buff," ");
  if(strncmp(token,"nx",2)==0){
    token=strtok(NULL," ");
    if(strncmp(token,"=",1)==0){
      token=strtok(NULL," ");
      len=strlen(token);
      *iflag=1;
      strncpy(value,token,len);
      return(0);
    }
    return(-1);
  }else if(strncmp(token,"ny",2)==0){
    token=strtok(NULL," ");
    if(strncmp(token,"=",1)==0){
      token=strtok(NULL," ");
      len=strlen(token);
      *iflag=2;
      strncpy(value,token,len);
      return(0);
    }
    return(-1);
  }else if(strncmp(token,"nz",2)==0){
    token=strtok(NULL," ");
    if(strncmp(token,"=",1)==0){
      token=strtok(NULL," ");
      len=strlen(token);
      *iflag=3;
      strncpy(value,token,len);
      return(0);
    }
    return(-1);
  }
  return(1);
}
/*-------------------------------------------------------------------------*/
int read_axis(char *buff)
{
  char *token;
  token=strtok(buff," ");
  if(strncmp(token,"axis",4)==0){
    token=strtok(NULL," ");
    if(strncmp(token,"=",1)==0){
      token=strtok(NULL," ");
      if(strncmp(token,"xy",2)!=0){
	printf("ERROR:: only axis=xy is allowed for PHITS output file\n");
	return(-1);
      }
      return(0);
    }
    return(-1);
  }
  return(1);
}
/*-------------------------------------------------------------------------*/
int read_mesh(char *buff)
{
  char *token;
  token=strtok(buff," ");
  if(strncmp(token,"mesh",4)==0){
    token=strtok(NULL," ");
    if(strncmp(token,"=",1)==0){
      token=strtok(NULL," ");
      if(strncmp(token,"xyz",3)!=0){
	printf("ERROR:: only mesh=xyz is allowed for PHITS output file\n");
	return(-1);
      }
      return(0);
    }
    return(-1);
  }
  return(1);
}
/*-------------------------------------------------------------------------*/
int read_xyzparam(char *buff, char *value, int *iflag)
{
  char *token;
  int len;
  token=strtok(buff," ");
  if(strncmp(token+1,"-type",5)==0){
    *iflag=0;
    token=strtok(NULL," ");
    if(strncmp(token,"=",1)==0){
      token=strtok(NULL," ");
      len=strlen(token);
      strncpy(value,token,len);
      return(0);
    }
    return(-1);
  }else if(strncmp(token+1,"min",3)==0){
    *iflag=1;
    token=strtok(NULL," ");
    if(strncmp(token,"=",1)==0){
      token=strtok(NULL," ");
      len=strlen(token);
      strncpy(value,token,len);
      return(0);
    }
    return(-1);
  }else if(strncmp(token+1,"max",3)==0){
    *iflag=2;
    token=strtok(NULL," ");
    if(strncmp(token,"=",1)==0){
      token=strtok(NULL," ");
      len=strlen(token);
      strncpy(value,token,len);
      return(0);
    }
    *iflag=0;
    return(-1);
  }
  return(1);
}
