*********************************************************************
      module readfile
*********************************************************************
      implicit none
      character(LEN=200) :: Cpath,Cdir
      integer :: Npath,Ndir,Nfile
      character(LEN=200),allocatable :: Cfilenames(:)
      real(8),allocatable :: zloc(:)
      contains
!----------------------------------------------------------------------
      subroutine READinputs(ixmin,ixmax,iymin,iymax,izmin,izmax,
     &     nxc,nyc,nzc,izstep,iorg,iparam,Cconvtable,Cinput)
      use stdinout, only:stdin,stdout
      implicit none
      integer,intent(out) :: ixmin,ixmax,iymin,iymax,izmin,izmax
      integer,intent(out) :: nxc,nyc,nzc
      integer,intent(out) :: izstep,iorg,iparam
      character(LEN=200),intent(out) :: Cconvtable,Cinput
      character(LEN=1024) :: dummy

      write(*,*)'Type DICOM2PHITS to run dicom2phits'
      read(stdin,'(a)') dummy
      if(dummy(1:11).ne.'DICOM2PHITS')then
       write(*,*)'ERROR:: signature DICOM2PHITS is not confirmed'
       write(*,*)'This input may not be for dicom2phits'
       stop
      endif
      write(*,*)'File for conversion of voxel data'
      read(stdin,*)Cconvtable
      write(*,*)'DICOM file directory'
      read(stdin,*)Cdir
      Ndir=len_trim(Cdir)
      write(*,*)'Directory for creating PHITS inputs'
      read(stdin,*)Cinput
      write(*,*)'Minimum slice number, Maximum slice number'
      read(stdin,*)izmin,izmax
      if(izmin.lt.1.or.izmax.lt.1)then
       write(stdout,*)'ERROR: Nzmin and Nzmax should be >= 1'
       stop
      endif
      izstep=1
      if(izmin.gt.izmax)then
       izstep=izmax
       izmax=izmin
       izmin=izstep
       izstep=-1
      endif
      write(*,*)'Clipping: Nxmin,Nxmax,Nymin,Nymax'
      read(stdin,*)ixmin,ixmax,iymin,iymax
      if(ixmin.lt.1)then
        write(stdout,*)'ERROR: Nxmin should be >= 1'
        stop
      elseif(ixmin.gt.ixmax)then
        write(stdout,*)'ERROR: Nxmin should be <= Nxmax'
        stop
      endif
      if(iymin.lt.1)then
        write(stdout,*)'ERROR: Nymin should be >= 1'
        stop
      elseif(iymin.gt.iymax)then
        write(stdout,*)'ERROR: Nymin should be <= Nymax'
        stop
      endif
      write(*,*)'Coase graning: Nxc, Nyc, Nzc'
      read(stdin,*)nxc,nyc,nzc
      if(nxc.lt.0.or.nyc.lt.0.or.nzc.lt.0)then
        write(stdout,*)'ERROR: Nxc, Nyc, Nzc should be >= 1'
        stop
      endif
      write(*,*)'Coordinates 0:zero at center 1:DICOM system'
      read(stdin,*)iorg
      if(iorg.lt.-1.or.iorg.gt.1)then
        write(stdout,*)'ERROR: choose coordinates',
     &      ' 0:Zero at center 1:DICOM system'
        stop
      endif
      write(*,*)'PHITS parameter: ',
     &     '1:PhotonTherapy 2:ParticleTherapy'
      read(stdin,*)iparam
      if(iparam.lt.1.or.iparam.gt.3)then
        write(stdout,*)'ERROR: choose parameter option ',
     &       '1:PhotonTherapy or 2:ParticleTherapy'
        stop
      endif

      return
      end subroutine READinputs
!----------------------------------------------------------------------
      subroutine SETreadfile(iz)
      integer,intent(in) :: iz
      character(LEN=1),parameter :: NULL=CHAR(0)
      Nfile=len_trim(Cfilenames(iz))
      Npath=Ndir+Nfile+1
      Cpath(1:Npath)=Cdir(1:Ndir)//Cfilenames(iz)(1:Nfile)//NULL

      return
      end subroutine SETreadfile
!----------------------------------------------------------------------
      subroutine READheader(izmin,nxnum,nynum,xdel,ydel,zdel,slope,hu0,
     &     xsh,xvec,yvec,zvec)
*     SliceThickness
*     SpacingBetweenSlices (not necessary)
*     Rows
*     Columns
*     PixelSpacing
*     RescaleIntercept
*     RescaleSlope
!----------------------------------------------------------------------
      implicit none
      integer,intent(in) :: izmin
      integer,intent(out) :: nxnum,nynum
      real(8),intent(out) :: xdel,ydel,zdel,slope,hu0
      real(8),intent(out) :: xsh(3),xvec(3),yvec(3),zvec(3)

      call SETreadfile(izmin)
      call c_get_header(nxnum,nynum,xdel,ydel,zdel,hu0,slope,
     &     xsh,xvec,yvec,Cpath)
      xdel=xdel*0.1d0
      ydel=ydel*0.1d0
      zdel=zdel*0.1d0
      xsh(1:3)=xsh(1:3)*0.1d0
      zvec(1)=xvec(2)*yvec(3)-xvec(3)*yvec(2)
      zvec(2)=xvec(3)*yvec(1)-xvec(1)*yvec(3)
      zvec(3)=xvec(1)*yvec(2)-xvec(2)*yvec(1)

      return
      end subroutine READheader
!----------------------------------------------------------------------
      subroutine READdirectory(izmax0,nlist)
!----------------------------------------------------------------------
      use stdinout, only:stdout
      implicit none
      integer,intent(in) :: izmax0
      integer,intent(out) :: nlist
      character(LEN=200) :: Cpath
      character(LEN=1),parameter :: NULL=CHAR(0)

      Npath=Ndir+1
      Cpath(1:Npath)=Cdir(1:Ndir)//NULL
      call c_get_files(nlist,Cpath)

      if(nlist.lt.izmax0)then
        write(stdout,'(a,i5)')
     &       ' ERROR: maximum slice number should be <= ',nlist
        stop
      endif
      return
      end subroutine READdirectory
!----------------------------------------------------------------------
      subroutine READfilenames(nlist)
!----------------------------------------------------------------------
      use stdinout, only:stdout
      implicit none
      integer,intent(in) :: nlist
      integer :: i

      open(20,file='tmp.txt',status='old')
      if(nlist.le.0)then
        write(stdout,'(2a)')' ERROR: no file is found in ',Cdir(1:Ndir)
        stop
      endif
      do i=1,nlist
        read(20,*)Cfilenames(i),zloc(i)
      enddo
      close(20,status='delete')

      return
      end subroutine READfilenames
!----------------------------------------------------------------------
      subroutine ALLOCATEfilenames(nlist)
!----------------------------------------------------------------------
      implicit none
      integer,intent(in) :: nlist
      allocate(Cfilenames(nlist),zloc(nlist))
      return
      end subroutine ALLOCATEfilenames
!----------------------------------------------------------------------
      subroutine DEALLOCATEfilenames
!----------------------------------------------------------------------
      implicit none
      deallocate(Cfilenames,zloc)
      return
      end subroutine DEALLOCATEfilenames
!----------------------------------------------------------------------
      end module readfile

